import ballerina.net.http;
import ballerina.file;

function testAddHeader (http:OutResponse res, string key, string value) (http:OutResponse) {
    res.addHeader(key, value);
    return res;
}

function testGetContentLength (http:OutResponse res) (int) {
    int length = res.getContentLength();
    return length;
}

function testGetHeader (http:OutResponse res, string key) (string) {
    string contentType = res.getHeader(key);
    return contentType;
}

function testGetHeaders (http:OutResponse res, string key) (string[]) {
    return res.getHeaders(key);
}

function testGetJsonPayload (http:OutResponse res) (json) {
    json payload = res.getJsonPayload();
    return payload;
}

function testGetProperty (http:OutResponse res, string propertyName) (string) {
    string payload = res.getProperty(propertyName);
    return payload;
}

function testGetStringPayload (http:OutResponse res) (string) {
    string payload = res.getStringPayload();
    return payload;
}

function testGetBinaryPayload (http:OutResponse res) (blob) {
    blob payload = res.getBinaryPayload();
    return payload;
}

function testGetXmlPayload (http:OutResponse res) (xml) {
    xml payload = res.getXmlPayload();
    return payload;
}

function testRemoveHeader (http:OutResponse res, string key) (http:OutResponse) {
    res.removeHeader(key);
    return res;
}

function testRemoveAllHeaders (http:OutResponse res) (http:OutResponse) {
    res.removeAllHeaders();
    return res;
}

function testSetHeader (string key, string value) (http:OutResponse) {
    http:OutResponse res = {};
    res.setHeader(key, value);
    return res;
}

function testSetJsonPayload (json value) (http:OutResponse) {
    http:OutResponse res = {};
    res.setJsonPayload(value);
    return res;
}

function testSetProperty (string name, string value) (http:OutResponse) {
    http:OutResponse res = {};
    res.setProperty(name, value);
    return res;
}

function testSetStringPayload (string value) (http:OutResponse) {
    http:OutResponse res = {};
    res.setStringPayload(value);
    return res;
}

function testSetXmlPayload (xml value) (http:OutResponse) {
    http:OutResponse res = {};
    res.setXmlPayload(value);
    return res;
}

function testSetEntityBody(file:File content, string contentType) (http:OutResponse) {
    http:OutResponse res = {};
    res.setEntityBody(content, contentType);
    return res;
}

@http:configuration{basePath:"/hello"}
service<http> helloServer {

    @http:resourceConfig {
        path:"/11"
    }
    resource echo1 (http:Connection conn, http:InRequest req) {
        http:OutResponse res = {};
        _ = conn.respond(res);
    }

    @http:resourceConfig {
        path:"/12/{phase}"
    }
    resource echo2 (http:Connection conn, http:InRequest req, string phase) {
        http:OutResponse res = {};
        res.reasonPhrase = phase;
        _ = conn.respond(res);
    }

    @http:resourceConfig {
        path:"/13"
    }
    resource echo3 (http:Connection conn, http:InRequest req) {
        http:OutResponse res = {};
        res.statusCode = 203;
        _ = conn.respond(res);
    }

    @http:resourceConfig {
        path:"/addheader/{key}/{value}"
    }
    resource addheader (http:Connection conn, http:InRequest req, string key, string value) {
        http:OutResponse res = {};
        res.addHeader(key, value);
        string result = res.getHeader(key);
        res.setJsonPayload({lang:result});
        _ = conn.respond(res);
    }

    @http:resourceConfig {
        path:"/getHeader/{header}/{value}"
    }
    resource getHeader (http:Connection conn, http:InRequest req, string header, string value) {
        http:OutResponse res = {};
        res.setHeader(header, value);
        string result = res.getHeader(header);
        res.setJsonPayload({value:result});
        _ = conn.respond(res);
    }

    @http:resourceConfig {
        path:"/getJsonPayload/{value}"
    }
    resource GetJsonPayload(http:Connection conn, http:InRequest req, string value) {
        http:OutResponse res = {};
        json jsonStr = {lang:value};
        res.setJsonPayload(jsonStr);
        json result = res.getJsonPayload();
        json lang = result.lang;
        res.setJsonPayload(lang);
        _ = conn.respond(res);
    }

    @http:resourceConfig {
        path:"/GetProperty/{key}/{value}"
    }
    resource GetProperty (http:Connection conn, http:InRequest req,string key, string value) {
        http:OutResponse res = {};
        res.setProperty(key, value);
        string property = res.getProperty(key);
        res.setJsonPayload({value:property});
        _ = conn.respond(res);
    }

    @http:resourceConfig {
        path:"/GetStringPayload/{valueStr}"
    }
    resource GetStringPayload(http:Connection conn, http:InRequest req, string valueStr) {
        http:OutResponse res = {};
        res.setStringPayload(valueStr);
        string value = res.getStringPayload();
        res.setStringPayload(value);
        _ = conn.respond(res);
    }

    @http:resourceConfig {
        path:"/GetXmlPayload"
    }
    resource GetXmlPayload(http:Connection conn, http:InRequest req) {
        http:OutResponse res = {};
        xml xmlStr = xml `<name>ballerina</name>`;
        res.setXmlPayload(xmlStr);
        xml value = res.getXmlPayload();
        string name = value.getTextValue();
        res.setStringPayload(name);
        _ = conn.respond(res);
    }

    @http:resourceConfig {
        path:"/RemoveHeader/{key}/{value}"
    }
    resource RemoveHeader (http:Connection conn, http:InRequest req, string key, string value) {
        http:OutResponse res = {};
        res.setHeader(key, value);
        res.removeHeader(key);
        var headerValue = res.getHeader(key);
        string header;
        if (headerValue == null) {
            header = "value is null";
        }
        res.setJsonPayload({value:header});
        _ = conn.respond(res);
    }

    @http:resourceConfig {
        path:"/RemoveAllHeaders"
    }
    resource RemoveAllHeaders (http:Connection conn, http:InRequest req) {
        http:OutResponse res = {};
        res.setHeader("Expect", "100-continue");
        res.setHeader("Range", "bytes=500-999");
        res.removeAllHeaders();
        var headerValue = res.getHeader("Range");
        string header;
        if (headerValue == null) {
            header = "value is null";
        }
        res.setJsonPayload({value:header});
        _ = conn.respond(res);
    }
}
